"""
@author: SunYanCN
@contact: sunyanhust@163.com
@blog: https://sunyancn.github.io
@version: 1.0
@license: MIT Licence
@file: config.py
@time: 2019-11-24 21:27:42
"""

DATASET_URL = {'msra_ner': "https://paddlehub-dataset.bj.bcebos.com/msra_ner.tar.gz",
               'chnsenticorp': "https://paddlehub-dataset.bj.bcebos.com/chnsenticorp.tar.gz",
               'lcqmc': 'https://paddlehub-dataset.bj.bcebos.com/lcqmc.tar.gz',
               'toxic': "https://bj.bcebos.com/paddlehub-dataset/toxic.tar.gz",
               }